CREATE SCHEMA system;
COMMENT ON SCHEMA system IS 'Database administration tools';

CREATE FUNCTION system.incarnation()        RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.reload()             RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.smart_shutdown()     RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.fast_shutdown()      RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.immediate_shutdown() RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.smart_restart()      RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.fast_restart()       RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION system.immediate_restart()  RETURNS CSTRING AS 'system.so' LANGUAGE C;
CREATE FUNCTION update_connect_file ()      RETURNS TRIGGER AS 'system.so' LANGUAGE C;

CREATE FUNCTION system.exists_database (NAME) RETURNS BOOLEAN AS '
  SELECT (SELECT COUNT(*) FROM pg_database WHERE datname = $1) = 1;
' LANGUAGE SQL STABLE;

CREATE FUNCTION system.exists_user (NAME) RETURNS BOOLEAN AS '
  SELECT (SELECT COUNT(*) FROM pg_user WHERE usename = $1) = 1;
' LANGUAGE SQL STABLE;

CREATE FUNCTION system.exists_ident (NAME) RETURNS BOOLEAN AS '
  SELECT (SELECT COUNT(*) FROM system.ident WHERE "map" = $1) = 1;
' LANGUAGE SQL STABLE;

CREATE TABLE system.ident (
  "id"      SERIAL8 NOT NULL UNIQUE PRIMARY KEY,
  "map"     NAME    NOT NULL CHECK (char_length("map") > 0),
  "os_user" NAME    NOT NULL CHECK (char_length("os_user") > 0),
  "pg_user" NAME    NOT NULL CHECK (system.exists_user("pg_user")),
  UNIQUE ("map", "os_user", "pg_user")
);

CREATE TABLE system.hba (
  "id"       SERIAL8     NOT NULL UNIQUE PRIMARY KEY,
  "type"     VARCHAR(16) NOT NULL DEFAULT 'host',
  "database" NAME        NOT NULL,
  "user"     NAME        NOT NULL,
  "host"     INET        NOT NULL DEFAULT '127.0.0.1',
  "method"   VARCHAR(16) NOT NULL DEFAULT 'md5',
  "option"   TEXT        NOT NULL DEFAULT '',
  UNIQUE ("type", "database", "user", "host"),
  CONSTRAINT valid_hba_type
    CHECK ("type" IN ('local', 'host', 'hostssl', 'hostnossl')),
  CONSTRAINT valid_hba_database
    CHECK ("database" IN ('all', 'sameuser', 'samerole')
           OR system.exists_database("database")),
  CONSTRAINT valid_hba_user
    CHECK ("user" = 'all' OR system.exists_user("user")),
  CONSTRAINT valid_hba_option
    CHECK (
      ("method" IN ('trust', 'reject', 'md5', 'crypt', 'password', 'krb5')
       AND (char_length("option") = 0)) OR
      ("method" = 'ident' AND system.exists_ident("option")) OR
      ("method" = 'ldap' AND (char_length("option") > 0)) OR
      ("method" = 'pam'))
);

INSERT INTO system.hba ("type","database","user","method")
  VALUES ('local','all','all','trust');

CREATE TRIGGER update_hba_file
  AFTER INSERT OR UPDATE OR DELETE ON system.hba
  FOR EACH STATEMENT EXECUTE PROCEDURE update_connect_file();
CREATE TRIGGER update_ident_file
  AFTER INSERT OR UPDATE OR DELETE ON system.ident
  FOR EACH STATEMENT EXECUTE PROCEDURE update_connect_file();
